import React from 'react'
import './styles/index.css';
import { app, api } from 'xadmin';

import i18n from 'xadmin-i18n'
import auth from 'xadmin-auth'
import form from 'xadmin-form'
import ui from 'xadmin-ui'
import model from 'xadmin-model'
import relate from 'xadmin-model/lib/relate'
import filter from 'xadmin-model/lib/filter'
import actions from 'xadmin-model/lib/actions'

import loading from 'xadmin-ui/lib/loading'
import splashscreen from 'xadmin-ui/lib/splashscreen'

import modalform from 'xadmin-model/lib/modalform'
import search from 'xadmin-model/lib/search'
import reldetail from 'xadmin-model/lib/reldetail'

import components from 'xadmin-antd'
//import components from 'xadmin-bootstrap'

import models from './models'
// import themes from './themes'
import API from './api'
import layout from './layout'
import icons from './icon'
import routers from './routers'

import 'moment/locale/zh-cn' 
import 'antd/dist/antd.min.css'
import logo from './icon/logo.svg'
import ServiceIcon from './icon/Service.svg'
import ModularIcon from './icon/Modular.svg'

import WinService from './models/Service/winIndex'
import Service from './models/Service/index'

app
.use(i18n)
.use(ui)
.use(loading)
.use(icons)
.use(form)
//.use(themes)
.use(model)
.use(filter)
.use(actions)
.use(relate)
.use(auth)
.use(modalform)
.use(search)
.use(reldetail)
.use(components)
.use(splashscreen)
.use({
  config: {
    api: API,
    locale: {
      lng: 'zh_Hans', moment: 'zh-cn'
    },
    auth: { 
      can_signin: false, 
      // can_signup: true, 
      can_reset_password: false, 
      persist_type: 'session-storage' 
    },
    date_format: {
      time: 'HH:mm:ss', date: 'YYYY-MM-DD', datetime: 'YYYY-MM-DD HH:mm:ss'
    },
    filter: {
      textDefaultSearch: true,
      submenuShowAllFilter: true
    },
    'site.title': <div className='Settings-title' style={{ cursor: 'pointer' }} onClick={ () => {app.go('/app/dashboard')} }>AIRIOT<br/>运维管理平台</div>,
    'site.logo': <img style={{ width: 45, height: 55 }} src={logo} onClick={ () => {app.go('/app/dashboard')} } />
  },
  components: layout.components,
  items: {
    menuItems: { type: 'array' }
  },
  menuItems: [
    { title: '服务管理', url: '/app/model/Service/list', iconComponent: ServiceIcon },
    { title: '模块管理', url: '/app/model/Modular/list', iconComponent: ModularIcon }
  ],
  reducers: {
    test: (state=0, action) => {
      if(action.type === 'TEST_ADD') return ++state
      return state
    }
  },
  mappers: {
    test: {
      method: {
        add: ({ dispatch }) => () => dispatch({ type: 'TEST_ADD' })
      }
    }
  },
  models,
  routers,
  context: app => (context, cb) => {
    const { store } = context
    api({name: 'api'}).fetch('/user/admin/check')
      .then(data => {
        
      }, err =>{
        if (err.status === 404) {
          localStorage.removeItem('user')
          app.go('/regest')
        } else {
          console.error(err)
        }
      })
    let userJson = localStorage.getItem('user')
    if (userJson) {
      const user = JSON.parse(userJson)
      store.dispatch({ type: '@@xadmin/AUTH_SIGN_IN', payload: user })
    }
    api({name: ''}).fetch(`api/server/info`, { method: 'GET' }).then(data => {
      if (data && data.json) { // { online: true/false, serverType: "windows/docker/k3s" }
        let result = {...data.json}
        if (data.json.serverType === 'windows') {
          result.url1 = 'win'
          result.url2 = 'win/pg'
          result.url3 = 'win/proc'
          result.url4 = 'win/offlineInstall'
          app.get('models').Service = WinService.Service
        } else if (data.json.serverType === 'docker') {
          result.url1 = 'docker'
          result.url2 = 'docker/compose'
          result.url3 = 'docker/containers'
          result.url4 = 'docker/install'
          app.get('models').Service = Service.Service
        }
        localStorage.setItem('serverInfo', JSON.stringify(result))
      }
      const url = data?.json?.repos ?
        data.json.repos.substring(data.json.repos.indexOf('repos')) : 'repos/latest'
      Promise.all([
        api({name: ''}).fetch(url, { method: 'GET' }), // 获取服务和模块的最新版本信息
      ]).then((data) => {
        if (data[0] && data[0].json) {
          context.newVersions = data[0].json
        }
        cb(null, context)
      }).catch(e => {
        context.newVersions = []
        context.serverInfo = {}
        cb(null, context)
      })
    })
  }
}).start({ container: '#root' })

