import { Layout, Menu, Card, Result } from 'antd'
import React from 'react'
import { use, config as _c, Block, app } from 'xadmin'
import { Icon } from 'xadmin-ui';

import { IsAuthenticated } from 'xadmin-auth'
import MainMenu from '../components/MainMenu'
import './../styles/layout.css'
const { Sider } = Layout

const ProtectError = ({ error, onClose }) => {
  return (
    <Card>
      <Result
        status="error"
        title="您没有查看该页面权限"
      >
      </Result>
    </Card>
  )
}

const App = IsAuthenticated(props => {
  const pathname = props.location.pathname
  const { user } = use('redux', state => ({ user: state.user }))
  const [collapseds, setCollapseds] = React.useState(false)
  const [selectedKeys, setSelectedKeys] = React.useState([])

  const onCollapse = (coll) => {
    setCollapseds(coll)
    localStorage.setItem('Menu_collapsed', coll)
  }

  React.useEffect(() => {
    const coll = localStorage.getItem('Menu_collapsed')
    setCollapseds(coll === "true" ? true : false)
  }, [])

  return (user.protectUrl && user.protectUrl != pathname) ? <ProtectError /> : (
    <Layout style={{ minHeight: '100vh', overflow: 'auto' }}>
      <Sider
        collapsible
        collapsed={collapseds}
        onCollapse={onCollapse}>

        <div className="logo" style={{ 'webkitAppRegion': 'drag' }}>
          {_c('site.logo')}
          {_c('site.title')}
        </div>

        <MainMenu name="mainmenu" theme="dark"
          isHome={false}
          setIsHome={() => { }}
          collapseds={collapseds}
          setSelectedKeys={setSelectedKeys}
          selectedKeys={pathname}
        />

        <Block name="top.right" >
          {
            items => <Menu selectedKeys={pathname}
              style={collapseds ? { marginBottom: 20, position: 'absolute', bottom: 30, left: 0 }
                : {}} theme="dark" >
              {items}
            </Menu>
          }
        </Block>
      </Sider>
      <div className="main-content" style={{ overflow: 'auto', width: '100%' }}>
        {props.children}
      </div>
    </Layout >
  )
})

export default App
