import React from 'react';
import { Button, message, Tooltip, Upload, Card, Popover, Popconfirm, Modal, Form, Row, Col } from 'antd';
import _ from 'lodash'
import moment from 'moment'
import { api, use, app } from 'xadmin'
import { Icon } from 'xadmin-ui'
import { DeleteOutlined, ArrowUpOutlined, DownloadOutlined, UploadOutlined } from '@ant-design/icons'
import { CheckProgress } from './../Service/components/addModel'

const OnlineUpgradeBtn = ({ image, containerID, latest, latestVersion, name, url }) => {
  const { modelDispatch } = use('model')
  const [loading, setLoading] = React.useState(false)
  const [version, setVersion] = React.useState(latestVersion)


  const onClick = () => {
    const str = image?.substring(image.lastIndexOf(':'), image.length)
    const newimg = image?.replace(str, ':' + version)
    window.open(url)
  }

  return (
    <Tooltip title={'下载'}>
      {/* <Popconfirm placement="topLeft" title={
        <div style={{ marginLeft: -20 }}>
          <span >版本信息:</span>
          <Input style={{ marginTop: 5 }} value={version} onChange={(e) => setVersion(e.target.value)} />
        </div>
      } icon={null} disabled={latest} onConfirm={onClick} okText="确定" cancelText="取消"> */}
        <DownloadOutlined disabled={!url} onClick={onClick} style={{ color: 'rgb(24 144 255)' }} />
      {/* </Popconfirm> */}
    </Tooltip>
  )
}

const UpgradeBtn = ({newVersion, version}) => {
  const [id, setId] = React.useState()
  const { modelDispatch } = use('model')
  const serverInfo = JSON.parse(localStorage.getItem('serverInfo'))
  // 是否能上网
  let ifOnline = serverInfo?.online
  const onClick = () => {
    // linux 接收 tar.gz 后缀文件，windows 接收 zip 后缀文件
    let url = newVersion?.assets?.[0]?.url
    api({ name: 'api' }).fetch(`/front/onlineInstall`, {
      method: 'POST',
      body: JSON.stringify({ url })
    }).then((data) => {
      if (data.status === 200 && data.json.id) {
        setId(data.json.id)
      } else {
        message.error('在线升级失败')
      }
    }).catch(e => {
      if (e.status === 400) {
        message.warning(e.message)
      } else {
        message.error('在线升级失败')
      }
    })
  }
  const installSuccess = () => {
    setId(null)
    modelDispatch({ type: 'GET_ITEMS' })
  }
  return ifOnline && newVersion?.assets?.[0]?.url && newVersion.tag_name !== ("v"+version) ? (
    <Popover
      content={(
        <CheckProgress id={id} index={null} installSuccess={installSuccess} type="front" >
          <Button onClick={onClick} >在线升级</Button>
        </CheckProgress>
      )}
      placement="right"
      trigger="click"
    >
      <Tooltip title="在线升级">
        <ArrowUpOutlined style={{ color: 'rgb(24 144 255)' }} />
      </Tooltip>
    </Popover>
  ) : null
}

const DeleteBtn = ({ item }) => {
  const { modelDispatch } = use('model')
  return (
    <Popconfirm onConfirm={() => {
      let url = `/front?packageName=${item.id}` + (item.item.groupName ? `&groupName=${item.item.groupName}` : '')
      api({ name: 'api' }).fetch(url, {
        method: 'DELETE'
      }).then(({ status }) => {
        if (status == 200) {
          message.success('删除成功')
          modelDispatch({ type: 'GET_ITEMS' })
        } else {
          message.error('删除失败')
        }
      })
    }} title="确认是否删除？" okText="确定" cancelText="取消">
        <Button type="primary" danger>
          <DeleteOutlined />
        </Button>
      </Popconfirm>
  )
}

const OnlineAddComponent = () => {
  const [serverList, setServerList] = React.useState(app.context.newVersions ? 
    app.context.newVersions.filter(item => item.group_name === 'front')
    : [])

  const installService = (item, index) => {
    let url = item.releases?.[0]?.assets?.[0]?.url
    api({ name: 'api' }).fetch(`/front/onlineInstall`, {
      method: 'POST',
      body: JSON.stringify({ url })
    }).then((data) => {
      if (data.status === 200 && data.json.id) {
        let newList = [...serverList]
        newList[index].loadingId = data.json.id
        setServerList(newList)
      } else {
        message.error('安装失败')
      }
    }).catch(e => {
      if (e.status === 400) {
        message.warning(e.message)
      } else {
        message.error('安装失败')
      }
    })
  }

  const installSuccess = (index) => {
    let newList = [...serverList]
    delete newList[index].loadingId
    setServerList(newList)
  }
  
  return (<Card className="service-add">
    <Row>
      {
        serverList.map((item, index) => (
          <Col span={6}>
            <Card>
              <span className="repo-name">{item.repo_name}</span>
              <span className="description">{item.description}</span>
              {
                item.releases && item.releases[0] ?
                (<>
                  <span className="tag-name">最新版本：{item.releases?.[0]?.tag_name}</span>
                  <span className="tag-name">最新更新时间：{moment(item.releases?.[0]?.create_time).format('YYYY-MM-DD HH:mm:ss')}</span>
                  <CheckProgress id={item.loadingId} index={index} installSuccess={installSuccess} type="front" >
                    <Button size="small" type="primary" onClick={() => installService(item, index)} style={{ marginTop: 10 }}>安装</Button>
                  </CheckProgress>
                </>)
                : <span className="tag-name">暂无版本</span>
              }
            </Card>
          </Col>
        ))
      }
    </Row>
  </Card>)
}

const OfflineAddComponent = () => {
  const [id, setId] = React.useState()
  const [loading, setLoading] = React.useState(false)
  const onSubmit = values => {
    if (values.file && values.file.file) {
      let formData = new FormData()
      formData.append('file', values.file.file)
      setLoading(true)
      fetch(`${api({name: 'api'}).getHost()}api/front/install`, {
        method: 'POST',
        body: formData,
        headers: {
          authorization: api({ name: 'api' }).headers.Authorization,
        },
      }).then(function(response) {
        setLoading(false)
        return response.json() // return到第二个then里面
      }).then((data) => {
        if (data.id) {
          setId(data.id)
        } else {
          message.error('安装失败')
        }
      }).catch(e => {
        setLoading(false)
        if (e.status === 400) {
          message.warning(e.message)
        } else {
          message.error('安装失败')
        }
      })
    } else {
      message.error('未发现有效文件')
    }
  }
  return (
    <Form onFinish={onSubmit}>
      <Form.Item
        label="模块上传"
        name="file"
        extra="上传文件"
        rules={[{ required: true, message: '至少需要上传一个文件!' }]}
        labelCol={{ span: 6 }}
        wrapperCol={{ span: 16 }}
      >
        <Upload beforeUpload={() => false} maxCount={1} >
          <Button type="primary" icon={<UploadOutlined />}>上传</Button>
        </Upload>
      </Form.Item>
      <Form.Item labelCol={{ span: 6 }} wrapperCol={{ span: 16, offset: 6 }}>
        <CheckProgress id={id} index={null} installSuccess={() => setId(null)} type="front" >
          <Button type="primary" loading={loading} htmlType="submit">提交</Button>
        </CheckProgress>
      </Form.Item>
    </Form>
  )
}

export { OnlineUpgradeBtn, UpgradeBtn, DeleteBtn, OnlineAddComponent, OfflineAddComponent }
