import React from 'react'
import { RelateAction } from 'xadmin-model/lib/relate'
import { Page, Icon, C } from 'xadmin-ui'
import { Button, message, Tooltip, Tabs, Card } from 'antd';
import { PlusOutlined } from '@ant-design/icons'
import { OnlineUpgradeBtn, DeleteBtn, OnlineAddComponent, OfflineAddComponent, UpgradeBtn } from './Buttons';
import { app, api, use } from 'xadmin';
import { Model, ModelBlock } from 'xadmin-model'
import ReactMarkdown from 'react-markdown'

const { TabPane } = Tabs

export default {
  Modular: {
    name: 'Modular',
    resource: 'api/front',
    type: 'object',
    icon: 'user', // fa-icon
    title: '模块管理',
    persistent: true,
    properties: {
      name: {
        type: 'string',
        title: "模块名称",
        column: { fixed: 'left' },
        //description: '用户的真实姓名'
        // renderTemplate: '<b style="color: <%= {1:"red", 2:"blue", 3:"green"}[item.type] || "" %>;"><%= value %></b>'
      },
      description:{
        type: 'string',
        title: '描述'
      },
      version: {
        type: 'string',
        title: '版本'
      },
      newVersion: {
        type: 'array',
        title: '最新版本'
      },
      website: {
        type: 'string',
        maxLength: 5
      },
      modTime: {
        type: 'string',
        title: '创建时间'
      },
    },
    permission: { view: true, add: true, delete: true },
    form: [ ],
    formEffect: form => {
      // form.useEffect(({ submitSucceeded, submitErrors, submitFailed }) => {
      //   console.log('submitSucceeded', submitSucceeded, submitErrors, submitFailed)
      // })
    },
    filters: {
      // submenu: [ 'name' ],
      //sidemenu: [ 'name' ]
    },
    itemActions: [ 
      (item) => { return <Button onClick={() => window.open(`http://www.airiot-edu.cn/portal/repos/${item.id}/releases`)}> 历史版本</Button>},
      (item) => (<DeleteBtn item={item} />)
    ],
    dataTableProps: (columns) => ({
      scroll: { x: 1500, y: document.body.clientHeight - 140 },
      columns: columns.map(c => c.key === '__action__' ? { ...c, width: 200 } : c )
    }),
    fieldRender: {
      version: ({ item }) => {
        return <span>v{item.version}</span>
      },
      newVersion: ({ item }) => {
        let newVersion = app?.context?.newVersions?.find(arg=> arg.repo_name == item.id)?.releases?.[0]?? {}
        let description = newVersion.description
        let url = app?.context?.newVersions?.find(arg=> arg.repo_name == item.id)?.releases?.[0]?.assets?.[0]?.url
        item.url = url
        return (<>
          <Tooltip 
            overlayClassName="newVersion" 
            color="white"
            title={() => <ReactMarkdown>{description}</ReactMarkdown>}
          ><span>{newVersion.tag_name} </span></Tooltip>
          <OnlineUpgradeBtn {...item} />
          <UpgradeBtn {...item} newVersion={newVersion} />
        </>)
      }
    },
    blocks:{},
    components: {
      DataForm: props => {
        return (
          <Card>
            <Tabs defaultActiveKey="1" >
              <TabPane tab="在线添加/更新" key="1">
                <OnlineAddComponent />
              </TabPane>
              <TabPane tab="离线添加/更新" key="2">
                <OfflineAddComponent />
              </TabPane>
            </Tabs>
          </Card>
        )
      },
      ListPage: props => {
        return (
        <Page title="模块管理" subTitle={<ModelBlock name="model.list.nav" />} actions={(
          <Button type="primary" onClick={() => app.go('/app/model/Modular/add')} >
            <PlusOutlined /> 添加/更新模块
          </Button>
        )}>
          <ModelBlock name="model.list.submenu" />
          <C is="Model.DataTable" />
        </Page>)
      }
    },
    batchActions: null,
    // editableFields: ['name', 'type'],
    batchChangeFields: ['website', 'brithday'],
    searchFields: [ 'name' ],
    required: [ 'name' ],
    readonly: [ 'name' ],
    listFields: [ 'name', 'modTime', 'description', 'version', 'newVersion' ],
    idProp: 'name',
    defaultPageSize: 10
  }
}
