import React from 'react'
import { api, use, app } from 'xadmin'
import { Button, Dropdown, message, Tooltip, Card, Popconfirm, Modal, Form, Menu, Popover } from 'antd';
import { CaretRightOutlined, PauseOutlined, ReloadOutlined, ArrowUpOutlined, DownSquareOutlined, DownloadOutlined, EditOutlined,
  SnippetsOutlined, InfoCircleOutlined, AreaChartOutlined, ConsoleSqlOutlined, UploadOutlined, DeleteOutlined } from '@ant-design/icons';
import UploadFile from './UploadFile'
import { CheckProgress } from './addModel'

const ActionsBtn = () => {
  const [Loading, setLoading] = React.useState({})

  const { selected } = use('model', state => ({ selected: state.selected }))
  const { modelDispatch } = use('model')
  const serverInfo = JSON.parse(localStorage.getItem('serverInfo'))
  let selecteds = selected

  const onClick = (url) => {
    if (selecteds && selecteds.length > 0) {
      setLoading({ [url]: true })
      Promise.all(selecteds.map(item => {
        return api({ name: 'api' }).fetch(`/${serverInfo?.url3}/${item.id}/${url}`, {
          method: 'POST',
        })
      })).then(data => {
        if (data && data.length > 0 && data.every(item => item.status === 200)) {
          message.success('操作成功')
          modelDispatch({ type: 'SELECT_CLEAR' })
          modelDispatch({ type: 'GET_ITEMS' })
          setLoading({ [url]: false })
        }
      }).catch(e => {
        message.error('操作失败')
        setLoading({ [url]: false })
        console.error(e)
      })
    } 
  }

  const deleteService = () => {
    if (selecteds && selecteds.length > 0) {
      setLoading({ delete: true })
      Promise.all(selecteds.map(item => {
        let url = serverInfo?.serverType === 'windows' ? 
            `/win/pg/service/${item.id}` : `/docker/compose/service/${item.name}`
        return api({ name: 'api' }).fetch(url, {
          method: 'DELETE',
        })
      })).then(data => {
        if (data && data.length > 0 && data.every(item => item.status === 200)) {
          message.success('删除成功')
          modelDispatch({ type: 'SELECT_CLEAR' })
          modelDispatch({ type: 'GET_ITEMS' })
          setLoading({ delete: false })
        }
      }).catch(e => {
        message.error('删除失败')
        setLoading({ delete: false })
        console.error(e)
      })
    } 
  }

  const btns = [
    { name: '开始', key: 'start', type: 'primary', icon: <CaretRightOutlined /> },
    { name: '停止', key: 'stop', type: '', icon: <PauseOutlined /> },
    // { name: '杀死', key: 'kill', type: '', icon: <CloseOutlined /> },
    { name: '重启', key: 'restart', type: '', icon: <ReloadOutlined /> },
  ]

  const disabledFn = (selecteds, item) => {
    if (selecteds.length === 0) return true
    // windows下按钮逻辑
    if (selecteds.every(item => ['running', 'retry wait'].indexOf(item.state) > -1) && item.key == 'start') return true
    if (selecteds.every(item => ['stopped', 'stopping'].indexOf(item.state) > -1) && item.key == 'stop') return true
    if (selecteds.every(item => ['retry wait'].indexOf(item.state) > -1) && item.key == 'restart') return true
    // linux下按钮逻辑
    if (selecteds.every(item => ['running', 'restarting', 'removing'].indexOf(item.state) > -1) && item.key == 'start') return true
    if (selecteds.every(item => ['stop', 'exited', 'created', 'removing'].indexOf(item.state) > -1) && item.key == 'stop') return true
    if (selecteds.every(item => ['restarting', 'removing'].indexOf(item.state) > -1) && item.key == 'restart') return true

    return false
  }

  return (
    <Card style={{ marginBottom: 10 }}>
      {
        btns.map(item => (
          <Button type={item.type} style={{ marginRight: 5 }}
            onClick={() => onClick(item.key)}
            loading={Loading[item.key]}
            disabled={disabledFn(selecteds, item)}
          >
            {Loading[item.key] || item.icon} {item.name}
          </Button>
        ))
      }
      <Popconfirm
        title="确认删除选中服务?"
        onConfirm={deleteService}
        okText="确认"
        cancelText="取消"
      >
        <Button type='primary' danger style={{ marginRight: 5 }}
              loading={Loading.delete}
              disabled={disabledFn(selecteds, {})}
            >
              {Loading.delete || <DeleteOutlined />} 删除
        </Button>
      </Popconfirm>
      {/* <span style={{ marginLeft: 10, color: '#ccc' }}>{disable ? '请先勾选要操作的容器' : `当前已选择${selectedLength}个容器`}</span> */}
    </Card>
  )
}

const ItemsBtn = ({ item, type }) => {
  const handleMenuClick = ({ key }) => {
    app.go(`/app/model/Service/${key}?podName=${item.id}`)
  }
  const serverInfo = JSON.parse(localStorage.getItem('serverInfo'))
  return (
    <>
      <Button icon={<DownSquareOutlined />} onClick={() => window.open(`http://www.airiot-edu.cn/portal/repos/${item.name}/releases`)}> 历史版本</Button>
      <OfflineBtn {...item} />
      <Button icon={<EditOutlined />} style={{ zIndex: 3 }} onClick={() => app.go(`/app/model/Service/${item.name}/edit`)}> 修改</Button>
      <Dropdown.Button style={{ marginLeft: -24, zIndex: 2 }} overlay={() => (
        <>
          <Menu onClick={handleMenuClick}>
            <Menu.Item key="log"><SnippetsOutlined /> 日志</Menu.Item>
            {
              type !== 'windows' && (<>
                <Menu.Item key={
                  serverInfo?.serverType === 'windows' ? "winStats" : "stats"
                }><AreaChartOutlined /> 图表</Menu.Item>
                <Menu.Item key="inspect"><InfoCircleOutlined /> 检查</Menu.Item>
                <Menu.Item key="console"><ConsoleSqlOutlined /> 控制台</Menu.Item>
              </>)
            }
          </Menu>
        </>
      )}></Dropdown.Button>
    </>
  )
}

const OfflineBtn = ({ item }) => {
  const [visible, setVisible] = React.useState(false)
  return (
    <>
      <Button icon={<UploadOutlined />} onClick={() => setVisible(true)}> 离线升级</Button>
      <Modal title="离线升级"
        visible={visible}
        footer={false}
        onCancel={() => setVisible(false)}
      >
        <Form
          labelCol={{ span: 4 }}
          wrapperCol={{ span: 14 }}
          layout="horizontal"
        >
          <Form.Item label="上传镜像">
              <UploadFile {...item} />
          </Form.Item>
        </Form>
      </Modal>
    </>
  )
}

// 下载按钮
const DownloadBtn = ({newPackage}) => {
  const serverInfo = JSON.parse(localStorage.getItem('serverInfo'))
  let packageUrl
  if (serverInfo?.serverType === 'windows' && newPackage?.assets){
    packageUrl = newPackage.assets.filter(item => item.url && item.url.indexOf('.zip') > -1)[0]?.url
  } else if(serverInfo?.serverType === 'docker' && newPackage?.assets) {
    packageUrl = newPackage.assets.filter(item => item.url && item.url.indexOf('.tar.gz') > -1)[0]?.url
  }
  const onClick = () => {
    window.open(packageUrl)
  }

  return packageUrl ? (
    <Tooltip title="下载">
      <DownloadOutlined  onClick={onClick} style={{ color: 'rgb(24 144 255)' }} />
    </Tooltip>
  ) : null
}

// 升级按钮
const UpgradeBtn = ({newPackage, version}) => {
  const [id, setId] = React.useState()
  const { modelDispatch } = use('model')
  const serverInfo = JSON.parse(localStorage.getItem('serverInfo'))
  // 是否能上网
  let ifOnline = serverInfo?.online
  const onClick = () => {
    // linux 接收 tar.gz 后缀文件，windows 接收 zip 后缀文件
    let url = serverInfo?.serverType === 'windows' ? 
      newPackage?.assets.filter(item => item.url && item.url.indexOf('.zip') > -1)[0]?.url
      : newPackage?.assets.filter(item => item.url && item.url.indexOf('.tar.gz') > -1)[0]?.url
    if (!url) {
      message.error('无对应服务安装包')
      return
    }
    api({ name: 'api' }).fetch(`/${serverInfo?.url1}/onlineInstall`, {
      method: 'POST',
      body: JSON.stringify({ url })
    }).then((data) => {
      if (data.status === 200 && data.json.id) {
        setId(data.json.id)
      } else {
        message.error('在线升级失败')
      }
    }).catch(e => {
      if (e.status === 400) {
        message.warning(e.message)
      } else {
        message.error('在线升级失败')
      }
    })
  }
  const installSuccess = () => {
    setId(null)
    modelDispatch({ type: 'GET_ITEMS' })
  }
  return ifOnline && newPackage?.assets?.[1]?.url && newPackage.tag_name !== version ? (
    <Popover
      content={(
        <CheckProgress id={id} index={null} installSuccess={installSuccess} >
          <Button onClick={onClick} >在线升级</Button>
        </CheckProgress>
      )}
      placement="right"
      trigger="click"
    >
      <Tooltip title="在线升级">
        <ArrowUpOutlined style={{ color: 'rgb(24 144 255)' }} />
      </Tooltip>
    </Popover>
  ) : null
}

export {
  ActionsBtn,
  ItemsBtn,
  OfflineBtn,
  DownloadBtn,
  UpgradeBtn
}