import React from 'react'
import { use } from 'xadmin'
import { SchemaForm } from 'xadmin-form';
import { Switch, Select, Input } from 'antd';

const { Option } = Select

const SocketForm = ({ connect }) => {

  const socketSchema = {
    type: 'object',
    properties: {
      cmd: {
        type: 'array',
        title: '命令',
        items: [{}]
      },
      tty: {
        type: 'boolean',
        title: '使用固定命令'
      },
      user: {
        type: 'string',
        title: '用户'
      }
    },
    form: [ 
      { key: 'cmd', component: CmdInput }, 
      { key: 'tty', component: TtyInput }, 
      'user'
    ],
    required: [ 'cmd', 'user' ]
  }

  return (<SchemaForm
    formKey={'setSocket'}
    schema={socketSchema}
    onSubmit={connect}
  />)
}

const CmdInput = ({ input }) => {
  const [tty, setTty] = React.useState(true)
  const { form } = use('form')

  React.useEffect(() => {
    form.useField('tty', state => {
      setTty(state.value)
      input.onChange(null)
    })
  }, [])

  const handleChange = value => {
    input.onChange([value])
  }
  
  return(<>
    {
      tty ?
      <Select onChange={handleChange}>
        <Option value='ash'>/bin/ash</Option>
        <Option value='bash'>/bin/bash</Option>
        <Option value='sh'>/bin/sh</Option>
      </Select>
      : <Input onChange={e => handleChange(e.target.value)} />
    }
  </>)
}

const TtyInput = ({ input }) => {
  React.useEffect(() => {
    input.onChange(false)
  }, [])
  return(<>
    <Switch onChange={checked => input.onChange(checked)} />
  </>)
}

export default SocketForm