import React, { Children } from 'react'
import { api, app } from 'xadmin'
import { Row, Col, Card, Button, message, Progress, Tag, Form, Upload } from 'antd'
import { SyncOutlined, UploadOutlined } from '@ant-design/icons';
import moment from 'moment'
import UploadFile from './UploadFile'
import ImageInput from './ImageInput'
import './../../../styles/service.css'

// 在线添加（服务）
const OnlineAddComponent = () => {
  const [serverList, setServerList] = React.useState(app.context.newVersions ? 
    app.context.newVersions.filter(item => item.group_name === 'server' || item.group_name === 'driver')
    : [])
  
  const installService = (item, index) => {
    const serverInfo = JSON.parse(localStorage.getItem('serverInfo'))
    // linux 接收 tar.gz 后缀文件，windows 接收 zip 后缀文件
    let url = serverInfo?.serverType === 'windows' ? 
      item.releases?.[0]?.assets.filter(item => item.url && item.url.indexOf('.zip') > -1)[0]?.url
      : item.releases?.[0]?.assets.filter(item => item.url && item.url.indexOf('.tar.gz') > -1)[0]?.url
    if (!url) {
      message.error('无对应服务安装包')
      return
    }
    api({ name: 'api' }).fetch(`/${serverInfo?.url1}/onlineInstall`, {
      method: 'POST',
      body: JSON.stringify({ url })
    }).then((data) => {
      if (data.status === 200 && data.json.id) {
        let newList = [...serverList]
        newList[index].loadingId = data.json.id
        setServerList(newList)
      } else {
        message.error('安装失败')
      }
    }).catch(e => {
      if (e.status === 400) {
        message.warning(e.message)
      } else {
        message.error('安装失败')
      }
    })
  }

  const installSuccess = (index) => {
    let newList = [...serverList]
    delete newList[index].loadingId
    setServerList(newList)
  }
  
  return (<Card className="service-add">
    <Row>
      {
        serverList.map((item, index) => (
          <Col span={6}>
            <Card>
              <span className="repo-name">{item.repo_name}</span>
              <span className="description">{item.description}</span>
              {
                item.releases && item.releases[0] ?
                (<>
                  <span className="tag-name">最新版本：{item.releases?.[0]?.tag_name}</span>
                  <span className="tag-name">最新更新时间：{moment(item.releases?.[0]?.create_time).format('YYYY-MM-DD HH:mm:ss')}</span>
                  <CheckProgress id={item.loadingId} index={index} installSuccess={installSuccess} >
                    <Button size="small" type="primary" onClick={() => installService(item, index)} style={{ marginTop: 10 }}>安装</Button>
                  </CheckProgress>
                </>)
                : <span className="tag-name">暂无版本</span>
              }
            </Card>
          </Col>
        ))
      }
    </Row>
  </Card>)
}

// 查看进度
const CheckProgress = ({id, index, installSuccess, children, type}) => {
  const checkInterval = React.useRef()
  const [inState, setInState] = React.useState({}) // download, install, run
  React.useEffect(() => {
    if (id) {
      checkInterval.current = setInterval(() => {
        api({ name: 'api' }).fetch(`/installInfo/${id}`, {
          method: 'GET'
        })
        .then((data) => {
          if (data.status === 200) {
            if (data.json.run) { // 运行启动
              if (data.json.run.progress === 100) { // 成功
                installSuccess(index)
                message.success(data.json.run.outInfo || '创建并启动服务成功！')
              } else if (data.json.run.err) {
                clearInterval(checkInterval.current)
                checkInterval.current = null
                installSuccess(index)
                message.error(data.json.run.err)
              } else {
                setInState({ type: '正在启动', progress: data.json.run.progress })
              }
            } else if (data.json.install) { // 安装
              if (data.json.install.progress === 100 && type === 'front') { // 前端成功（没有启动）
                installSuccess(index)
                message.success(data.json.install.outInfo || '更新模块成功！')
              } else if  (data.json.install.err) {
                clearInterval(checkInterval.current)
                checkInterval.current = null
                installSuccess(index)
                message.error(data.json.install.err)
              } else {
                setInState({ type: '正在安装', progress: data.json.install.progress })
              }
            } else if (data.json.download) { // 下载
              if (data.json.download.err) {
                clearInterval(checkInterval.current)
                checkInterval.current = null
                installSuccess(index)
                message.error(data.json.download.err)
              } else {
                setInState({ type: '正在下载', progress: data.json.download.progress })
              }
            } else {
              setInState({})
            }
          }
        })
      }, 1000)
    } else if (checkInterval.current) {
      clearInterval(checkInterval.current)
      checkInterval.current = null
    }
    return () => {
      clearInterval(checkInterval.current)
    }
  }, [id])
  return id ? 
    (<>
      <Tag color="blue">{ inState.type?? '正在下载' }</Tag>
      <SyncOutlined style={{ width: 14, height: 14, marginTop: 19 }} spin />
      <Progress percent={inState.progress?? 0} status="active" />
    </>)
    : children
}

// 离线添加（服务）

const OfflineAddComponent = () => {
  const [id, setId] = React.useState()
  const onSubmit = values => {
    if (values.file && values.file.file) {
      let formData = new FormData()
      formData.append('file', values.file.file)
      const serverInfo = JSON.parse(localStorage.getItem('serverInfo'))
      fetch(`${api({name: 'api'}).getHost()}api/${serverInfo?.url4}`, {
        method: 'POST',
        body: formData,
        headers: {
          authorization: api({ name: 'api' }).headers.Authorization,
        },
      }).then(function(response) {
        return response.json() // return到第二个then里面
      }).then((data) => {
        if (data.id) {
          setId(data.id)
        } else {
          message.error('安装失败')
        }
      }).catch(e => {
        if (e.status === 400) {
          message.warning(e.message)
        } else {
          message.error('安装失败')
        }
      })
    } else {
      message.error('未发现有效文件')
    }
  }
  return (
    <Form onFinish={onSubmit}>
      <Form.Item
        label="服务上传"
        name="file"
        extra="上传文件"
        rules={[{ required: true, message: '至少需要上传一个文件!' }]}
        labelCol={{ span: 6 }}
        wrapperCol={{ span: 16 }}
      >
        <Upload beforeUpload={() => false} maxCount={1} >
          <Button type="primary" icon={<UploadOutlined />}>上传</Button>
        </Upload>
      </Form.Item>
      <Form.Item labelCol={{ span: 6 }} wrapperCol={{ span: 16, offset: 6 }}>
        <CheckProgress id={id} index={null} installSuccess={() => setId(null)} >
          <Button type="primary" htmlType="submit">提交</Button>
        </CheckProgress>
      </Form.Item>
    </Form>
  )
}

// 高级添加（服务）（Linux）
const highAddSchema = {
  name: 'docker',
  title: '表单创建',
  type: 'object',
  properties: {
    name: {
      type: 'string',
      title: '应用名称',
      description: '具有此值的“app”标签将添加到已部署的 Deployment 和 Service 中'
    },
    image: {
      type: 'string',
      title: '容器镜像',
      description: '输入公共镜像的 URL，格式：xxx:xxx'
    },
    // replicas: {
    //   type: 'number',
    //   title: 'pod的数量',
    //   description: 'Deployment 将跨集群创建 pod 以维护所需数量'
    // },
    service: {
      type: 'string',
      title: 'service类型',
      enum: ['None', 'Internal', 'External'],
      // description: '可选，可以定义内部或外部 Service，将传入端口映射到容器的目标端口'
    },
    ports: {
      type: 'array',
      title: '端口',
      items: {
        type: 'object',
        properties: {
          Host: {
            type: 'string',
            title: '主机',
          },
          Container: {
            type: 'string',
            title: '容器',
          },
          Protocol: {
            type: 'string',
            title: '协议',
            enum: ['tcp', 'udp',]
          },
        },
        required: ['Host', 'Container', 'Protocol']
      }
    },
    path: {
      type: 'string',
      title: '接口统一路径',
    },
    // description: {
    //   type: 'string',
    //   title: '应用描述',
    //   fieldType: 'textarea',
    // },
    labels: {
      type: 'array',
      title: '标签',
      items: {
        type: 'object',
        properties: {
          key: {
            type: 'string',
            title: '值',
          },
          value: {
            type: 'string',
            title: '名称',
          }
        },
        required: ['key', 'value']
      },
      // description: '至少创建标签：key: app，value：应用名称'
    },
    volumes: {
      type: 'array',
      title: '数据卷',
      items: {
        type: 'object',
        properties: {
          Container: {
            type: 'string',
            title: '容器',
          },
          Host: {
            type: 'string',
            title: '主机',
          },
          Mode: {
            type: 'string',
            title: '模式',
            enum: ['volume', 'bind', 'ro']
          }
        },
        required: ['Container', 'Host']
      }
    },
    // cpuRequirement: {
    //   type: 'number',
    //   title: 'CPU 最低需求 (cores)',
    //   description: '您可以指定容器的最低 CPU 和内存 requirements'
    // },
    // memoryRequirement: {
    //   type: 'number',
    //   title: 'Memory 最低需求 (MiB)',
    // },
    command: {
      type: 'string',
      title: '运行命令',
    },
    networks: {
      type: 'string',
      title: '网络',
    },
    privileged: {
      type: 'boolean',
      title: '特权',
      description: '特权容器中的进程等同于在主机上以root身份运行的进程'
    },
    environment: {
      type: 'array',
      title: '环境变量',
      items: {
        type: 'object',
        properties: {
          name: {
            type: 'string',
            title: '名字',
          },
          value: {
            type: 'string',
            title: '值',
          },
        },
        required: ['name', 'value']
      },
      description: '可在容器中使用的环境变量。值可以使用 $(VAR_NAME) 语法引用其他变量'
    },
    file: {
      type: 'object',
      title: '镜像文件',
    }
  },
  form: [
    'name', 
    { key: 'image', component: ImageInput },
    'service', 'path', 'ports', 'labels', 'volumes', 'command', 'networks', 'privileged', 'environment',
    // { key: 'description', type: 'textarea', attrs: { rows: 2 } },
    { key: 'file', component: UploadFile },
  ],
  formEffect: (form, a) => {
    // form.change('replicas', 1)
    form.setFieldData('path', { display: false })
    form.setFieldData('ports', { display: false })
    form.useField('service', state => {
      if (state.value === 'Internal' || state.value === 'External') {
        form.setFieldData('path', { display: true })
        form.setFieldData('ports', { display: true })
        // form.change('path', null)
        // form.change('ports', null)
      } else {
        form.setFieldData('path', { display: false })
        form.setFieldData('ports', { display: false })
        form.change('path', '')
        form.change('ports', [])
      }
    })
  },
  required: ['name', 'image', 'service', 'path', 'ports']
}

// 高级添加（服务）（windows）
const hignAddWinSchema = {
  name: 'docker',
  title: '表单创建',
  type: 'object',
  properties: {
    name: {
      type: 'string',
      title: '应用名称'
    },
    command: {
      type: 'string',
      title: '命令',
    },
    directory: {
      type: 'string',
      title: '目录',
    },
    startRetries: {
      type: 'number',
      title: '失败重试次数',
      description: '-1为持续不停重试'
    },
    startAuto: {
      type: 'boolean',
      title: '自动启动',
    }
  },
  form: ['name', 'command', 'directory', 'startRetries', 'startAuto'],
  required: ['name', 'command', 'directory', 'startRetries']
}
  
export {
  OnlineAddComponent,
  OfflineAddComponent,
  CheckProgress,
  highAddSchema,
  hignAddWinSchema
}
