import React from 'react'
import { RelateAction } from 'xadmin-model/lib/relate'
import { Tag, Button, message, Tooltip, Popconfirm, Tabs, Card, Table } from 'antd'
import { ActionsBtn, ItemsBtn, DownloadBtn, UpgradeBtn } from './components/Buttons'
import { api, app, use } from 'xadmin'
import { Page, Icon, C } from 'xadmin-ui'
import { ModelBlock } from 'xadmin-model'
import { SchemaForm } from 'xadmin-form';
import moment from 'moment'
import { HasPermission } from 'xadmin-auth'
import { OnlineAddComponent, OfflineAddComponent, highAddSchema } from './components/addModel'
import { CaretRightOutlined, PauseOutlined, ReloadOutlined, DeleteOutlined, PlayCircleOutlined,
  DownloadOutlined, PlusOutlined } from '@ant-design/icons';
import ReactMarkdown from 'react-markdown'

const { TabPane } = Tabs

export default {
  Service: {
    name: 'Service',
    resource: `api/docker/compose/service`,
    type: 'object',
    icon: 'user', // fa-icon
    title: '服务管理',
    persistent: true,
    properties: {
      name: {
        type: 'string',
        title: '服务名称',
        column: { fixed: 'left' },
        // description: '用户的真实姓名'
        // renderTemplate: '<b style="color: <%= {1:"red", 2:"blue", 3:"green"}[item.type] || "" %>;"><%= value %></b>'
      },
      state: {
        type: 'string',
        title: '状态',
        width: 100
      },
      createTime: {
        type: 'string',
        title: '创建时间'
      },
      image: {
        type: 'string',
        title: '镜像'
      },
      port: {
        type: 'string',
        title: '发布端口',
        width: 100
      },
      version: {
        type: 'string',
        title: '当前版本'
      },
      newVersion: {
        type: 'string',
        title: '最新版本',
        width: 130
      }
    },
    permission: { view: true, add: true, edit: true },
    form: [ 'name', 'state', 'createTime', 'image', 'port', 'version' ],
    formEffect: form => {

      form.useEffect(({ submitSucceeded, submitErrors, submitFailed }) => {
        console.log('submitSucceeded', submitSucceeded, submitErrors, submitFailed)
      }, [ 'submitSucceeded', 'submitErrors', 'submitFailed' ])
    },
    filters: {
      // submenu: [ 'name' ],
      // sidemenu: [ 'name' ]
    },
    itemActions: [
      ({ item }) => (<ItemsBtn item={item} />)
    ],
    dataTableProps: (columns) => ({
      scroll: { x: 1400, y: document.body.clientHeight - 210 },
      columns: columns.map(c => c.key === '__action__' ? { ...c, width: 300 } : c )
    }),
    blocks: {
      'model.list.submenu': (props) => <ActionsBtn {...props} />,
    },
    components: {
      DataForm: props => {
        // 修改页面没有名称字段
        const getEditSchema = () => {
          let result = { ...highAddSchema }
          result.form = [ ...highAddSchema.form.slice(1) ]
          result.required = [ ...highAddSchema.required.slice(1) ]
          return result
        }
        const schema = props.id ? getEditSchema() : highAddSchema
        
        // 修改页面初始值参数处理
        const getInitialValues = () => {
          if (!props.id) return {}

          let result = { ...props.item }
          result.image = result.image ? result.image.Name + ':' + result.image.Tag : null
          result.command = result.command ? result.command[0] : null
          result.environment = result.environment ? result.environment.map(item => {
            return { name: item.split('=')[0], value: item.split('=')[1] }
          }) : null
          result.labels = result.labels ? result.labels.map(item => {
            return { key: item.split('=')[0], value: item.split('=')[1] }
          }) : null
          result.networks = result.networks ? result.networks[0] : null

          return result
        }
        const initialValues = getInitialValues()

        const onSubmit = value => {
          let params = {
            "image": {
              "name": value.image.split(':')[0],
              "tag": value.image.split(':')[1]
            },
            "name": value.name,
            "service": value.service
          }
          if (value.command) params.command = [value.command]
          if (value.environment) params.environment = value.environment.map(item => item.name + '=' + item.value)
          if (value.labels) params.labels = value.labels.map(item => item.key + '=' + item.value)
          if (value.networks) params.networks = [value.networks]
          if (value.privileged) params.privileged = value.privileged
          if (value.volumes) params.volumes = value.volumes
          if (value.service === 'Internal' || value.service === 'External') {
            params.path = value.path
            params.ports = value.ports
          }
          if (props.id) { // 修改
            delete params.id
            delete params.version
            api({ name: 'api' }).fetch(`/docker/compose/service/${params.name}`, {
              method: 'PUT',
              body: JSON.stringify(params)
            })
            .then(({ status }) => {
              if (status === 200) {
                message.success('修改服务成功')
                app.go('/app/model/Service/list')
              }
            }).catch(e => {
              message.error(e.message)
            })
          } else { // 新增
            api({ name: 'api' }).fetch(`/docker/compose/service`, {
              method: 'POST',
              body: JSON.stringify(params)
            })
            .then(({ status }) => {
              if (status === 200) {
                message.success('新增服务成功')
                app.go('/app/model/Service/list')
              }
            }).catch(e => {
              message.error(e.message)
            })
          }
        }
        const onCloseModel = () => {}
        return props.id ? (
          <Card>
            <SchemaForm
              key="service.params.form"
              formKey="service.params"
              schema={schema}
              style={{ position: 'absolute' }}
              initialValues={initialValues}
              onSubmit={onSubmit}
              onClose={onCloseModel}
              // component={FormLayout} 
            />
          </Card>
        ) : (
          <Card>
            <Tabs defaultActiveKey="1" >
              <TabPane tab="在线添加" key="1">
                <OnlineAddComponent />
              </TabPane>
              <TabPane tab="离线添加" key="2">
                <OfflineAddComponent />
              </TabPane>
              <TabPane tab="高级添加" key="3">
                <SchemaForm
                  key="service.params.form"
                  formKey="service.params"
                  schema={schema}
                  style={{ position: 'absolute' }}
                  initialValues={initialValues}
                  onSubmit={onSubmit}
                  onClose={onCloseModel}
                  // component={FormLayout} 
                />
              </TabPane>
            </Tabs>
          </Card>
        )
        // return (<C is="Model.TabsForm" schemaKey="service.schema" forceTabs={true} baseTitle="输入并创建" {...props} />)
      },
      ListPage: props => {
        const [loading, setLoading] = React.useState(false)
        const { modelDispatch } = use('model')

        const operateAll = (name) => {
          setLoading(true)
          api({ name: 'api' }).fetch(`/docker/compose/${name}`, {
            method: 'POST',
          }).then(({ status }) => {
            if (status === 200) {
              message.success('操作成功')
              modelDispatch({ type: 'GET_ITEMS' })
            } else {
              message.error('操作失败')
            }
            setLoading(false)
          }).catch(e => {
            message.error('操作失败')
            console.error(e)
            setLoading(false)
          })
        }
        return (
          <Page title="服务管理"
            subTitle={<ModelBlock name="model.list.nav" />}
            actions={<>
              <Tooltip title="启动全部服务">
                <Button loading={loading} onClick={() => operateAll('start')} >
                  <CaretRightOutlined /> 启动
                </Button>
              </Tooltip>
              <Tooltip title="重启全部服务">
                <Button loading={loading} onClick={() => operateAll('restart')} >
                  <ReloadOutlined /> 重启
                </Button>
              </Tooltip>
              <Tooltip title="停止全部服务">
                <Button loading={loading} onClick={() => operateAll('stop')} >
                  <PauseOutlined /> 停止
                </Button>
              </Tooltip>
              <Tooltip title="停止并删除全部服务">
                <Popconfirm title="确认停止并删除全部服务？" onConfirm={() => operateAll('down')} >
                  <Button loading={loading} >
                    <DeleteOutlined /> 删除
                  </Button>
                </Popconfirm>
              </Tooltip>
              <Tooltip title="创建并启动全部服务">
                <Button loading={loading} onClick={() => operateAll('up')} >
                  <PlayCircleOutlined /> 创建启动
                </Button>
              </Tooltip>
              <Button type="primary" onClick={() => app.go('/app/model/Service/depolyment')} >
                编辑部署文件
              </Button>
              {/* <HasPermission permission="docker.add"> */}
                <Button type="primary" onClick={() => app.go('/app/model/Service/add')} >
                  <PlusOutlined /> 添加服务
                </Button>
              {/* </HasPermission> */}
              <ModelBlock name="model.list.navbtn" />
            </>}
          >
            <ModelBlock name="model.list.submenu" />
            <C is="Model.DataTable" size="small" />
          </Page>
        )
      }
    },
    fieldRender: {
      state: ({ value }) => {
        let keyList = {
          'running': { title: '运行', color: 'blue' },
          'exited': { title: '停止', color: '#a35f68' },
          'stop': { title: '停止', color: '#a35f68' },
          'created': { title: '已创建', color: 'green' },
          'paused': { title: '暂停', color: 'red' },
          'restarting': { title: '重启中', color: 'yellow' },
          'dead': { title: '启动失败', color: 'red' },
          'start': { title: '启动', color: 'blue' },
          'removing': { title: '正在删除', color: '#a35f68' },
        }
        return <Tag color={keyList[value]?.color}>{keyList[value]?.title}</Tag>
      },
      createTime: ({ value }) => {
        return moment(value*1000).format('YYYY-MM-DD HH:mm:ss')
      },
      newVersion: ({ item }) => {
        let newVersion = '', description = ''
        if (app.context.newVersions) {
          app.context.newVersions.forEach(item1 => {
            if ((item1.group_name === 'server' || item1.group_name === 'driver') && item1.repo_name === item.repo) {
              newVersion = item1.releases?.[0]?.tag_name
              description = item1.releases?.[0]?.description
            }
          })
        }
        let newPackage = {}
        if (app.context.newVersions) {
          app.context.newVersions.forEach(item1 => {
            if ((item1.group_name === 'server' || item1.group_name === 'driver') && item1.repo_name === item.repo) {
              newPackage = item1.releases?.[0]?? {}
            }
          })
        }
        return (<>
          <Tooltip 
            overlayClassName="newVersion" 
            placement="left"
            color="white"
            title={() => <ReactMarkdown>{description}</ReactMarkdown>}
          ><span>{newVersion} </span></Tooltip>
          <DownloadBtn {...item} newPackage={newPackage} /> 
          <UpgradeBtn {...item} newPackage={newPackage} />
        </>)
      }
    },
    // batchActions: null,
    // editableFields: ['name', 'type', 'address.street'],
    batchChangeFields: ['website', 'brithday', 'address.street'],
    searchFields: [ 'name' ],
    required: [ 'name', 'image' ],
    readonly: [ 'id' ],
    listFields: [ 'name', 'state', 'version', 'newVersion', 'image', 'port', 'createTime' ],
    defaultPageSize: 20
  }
}
