import React from 'react'
import { api, app } from 'xadmin';
import { Page } from 'xadmin-ui';
import _ from 'lodash';
import { Card, Alert, Button } from 'antd'
import { ArrowLeftOutlined } from '@ant-design/icons';
import SocketForm from './../components/SocketForm'

import { Terminal } from 'xterm';
import { FitAddon } from 'xterm-addon-fit'
import { AttachAddon } from 'xterm-addon-attach'

import 'xterm/css/xterm.css'

const DeviceLog = React.forwardRef(({ setState, socketId }, ref) => {

  const logsRef = React.useRef()
  const term = React.useRef()
  const sock = React.useRef()

  React.useEffect(() => {
    if (!socketId) return
    term.current = new Terminal({
      fontSize: 14
    })
    if (!sock.current) {
      sock.current = createSocket(socketId)
    }
    const attachAddon = new AttachAddon(sock.current)
    const fitAddon = new FitAddon()
    term.current.loadAddon(attachAddon)
    term.current.loadAddon(fitAddon)
    term.current.open(logsRef.current)
    fitAddon.fit()
    term.current.focus()

  }, [socketId])

  React.useImperativeHandle(ref, () => ({
    closeWS:()=>{
      sock.current && sock.current.close()
    }
  }))

  const createSocket = (id) => {
    const user = localStorage.getItem('user') ? JSON.parse(localStorage.getItem('user')) : {}
    const token = user && user.accessToken
    let url

    if (process.env.NODE_ENV === 'development') { // 开发环境
      url = `ws://iot.htkjbjf.com:10090/api/websocket/exec?id=${id}&token=${token}`
    } else {
      url = 'ws://' + window.document.location.origin.substring(7) +`/api/websocket/exec?id=${id}&token=${token}`
    }

    let socket = new WebSocket(url)
    socket.onopen = () => {
      console.log('WebSocket已链接!')
      setState('connect')
    }
    socket.onmessage = msg => {}
    socket.onclose = v => {
      console.log('WebSocket已关闭')
      setState('unconnect')
    }
    socket.onerror = err => {
      console.error('WebSocket连接错误', err)
      setState('unconnect')
    }

    return socket
  }

  return <div style={{ width: '100%', height: '100%' }} ref={logsRef}></div>
})

const ConsolePage = (props) => {
  const { location: { query } } = props
  const podName = query && query.podName
  const [socketId, setSocketId] = React.useState()
  const [state, setState] = React.useState('unconnect') // 'connect','connecting','unconnect'
  const [params, setParams] = React.useState({})
  const deviceLogRef = React.useRef()

  const connect = (params) => {
    setParams(params)
    let body = {
      "attachStderr": true,
      "attachStdin": true,
      "attachStdout": true,
      ...params
    }
    api({ name: 'api' }).fetch(`/docker/containers/${podName}/exec`, {
      method: 'POST',
      body: JSON.stringify(body)
    }).then(({status, json}) => {
      if (status === 200) {
        setSocketId(json.Id)
        setState('connect')
      }
    })
  }

  const unconnect = () => {
    deviceLogRef.current.closeWS()
    setState('unconnect')
  }

  return (
    <Page title={<span><ArrowLeftOutlined onClick={() => app.go('/app/model/Service/list')} /> 控制台</span>}>
      {
        state === 'unconnect' ?
        <Card bordered={false}>
          <SocketForm connect={connect} />
        </Card>
        :
        <>
          <Alert type="success"
            message={'用户 <'+params.user+'> 在使用 <'+params.cmd[0]+'> 命令'}
            action={
              <Button size="small" type="ghost" onClick={unconnect}>
                断开连接
              </Button>
            }
          />
          <Card bordered={false} style={{ height: '100%', display: 'flex', flexDirection: 'column' }}
            bodyStyle={{ flexGrow: 1, display: 'flex', flexDirection: 'column' }}>
            <DeviceLog ref={deviceLogRef} setState={setState} socketId={socketId} />
          </Card>
        </>
      }
    </Page>
  )
}

export default ConsolePage