import React from 'react';
import { api, app } from 'xadmin';
import { Loading, Page } from 'xadmin-ui';
import { Card, Radio, Tree } from 'antd'
import { ArrowLeftOutlined } from '@ant-design/icons';
import _ from 'lodash';

const InspectPage = (props) => {

  const { location: { query } } = props
  const id = query && query.container_id
  const podName = query && query.podName
  const [loading, setLoading] = React.useState(false)
  const [treeData, setTreeData] = React.useState(false)
  const [json, setJson] = React.useState(false)

  const queryData = () => {
    setLoading(true)
    api({ name: `api/docker/containers/${podName}` }).fetch('', {})
      .then(({ json, body }) => {
        setTreeData(objTree(json.Id, json))
        const temp = body.replace(/,/g, ',\n    ').replace(/{/g, '{\n    ').replace(/},/g, '\n    },')
        setJson(temp)
        setLoading(false)
      })
  }

  React.useEffect(() => queryData(), [])

  const getType = (item) => {
    return Object.prototype.toString.apply(item).slice(8, -1);
  }

  const objTree = (key, val) => {
    let i = 0
    const expandObj = (label, value) => {
      if (getType(value) === 'Object' || getType(value) === 'Array') {
        // 对象，依次拓展
        let result = [];
        for (let key of Object.keys(value)) {
          result.push(expandObj(key, value[key]));
        }
        i++
        return {
          title: `${label}`,
          key: `${i}`,
          children: result
        };
      } else {
        i++
        // 若是普通类型，返回正常对象
        return {
          title: `${label}：${value}`,
          key: `${i}`,
          isLeaf: true
        };
      }
    }
    return [expandObj(key, val)]
  }


return (
  <Page title={<span><ArrowLeftOutlined onClick={() => app.go('/app/model/Service/list')} /> 检查容器</span>}>
    <LogFilter id={id} loading={loading} treeData={treeData} json={json}/>
  </Page>
)
}

const LogFilter = ({ loading, treeData, json }) => {
  const [type, setType] = React.useState('tree')

  return (
    <Card title={<div style={{ display: 'flex', justifyContent: ' space-between', }}>
      <span>检查</span>
      <Radio.Group
        options={[
          { label: '树状形式', value: 'tree' },
          { label: '文档形式', value: 'text' }
        ]}
        onChange={(e) => setType(e.target.value)}
        value={type}
        optionType="button"
      />
    </div>}>
      {loading ? <Loading /> : (type === 'tree' ? <Tree treeData={treeData} defaultExpandedKeys={[treeData[0]?.key]} ></Tree> : <pre>&nbsp;&nbsp;{json}</pre>)}
    </Card>
  )
}

export default InspectPage
