import { Button, Card, DatePicker, Form as AntdForm, Input, Switch } from 'antd';
import { ArrowLeftOutlined } from '@ant-design/icons';
import moment from 'moment';
import React from 'react';
import { api, app } from 'xadmin';
import { Form } from 'xadmin-form';
import { Loading, Page } from 'xadmin-ui';

const { TextArea } = Input


const DatePickerInput = (props) => {
  const { input, field } = props
  // const format = field.datetimeFormat || 'YYYY-MM-DD HH:mm:ss'

  const getValue = (value) => {
    return new Date(value).getTime()/1000
  }
  const onChange = (value) => {
    input.onChange(getValue(value))
  }
  return <DatePicker showTime onChange={onChange} value={input.value ? moment(input.value*1000) : null} placeholder="选择日期" />
}

const Switchs = (props) => {
  const { input } = props
  const [ value, setValue ] = React.useState(input.value)
  const onChange = (val) => {
    setValue(val)
    input.onChange(val)
  }
  return (<Switch checked={value} onChange={onChange} />)
}

const FormLayout = props => {
  const { children, invalid, handleSubmit, reset, onDelete, schema, submitting, onClose } = props
  return (
    <AntdForm onSubmit={handleSubmit}>
      <div>{children}</div>
    </AntdForm>
  )
}

const LogPage = (props) => {

  const { location: { query } } = props
  const id = query && query.container_id
  const podName = query && query.podName
  const [filter, setFilter] = React.useState({ 'timestamps': false, 'tail': 100, 'refresh': true })
  const [body, setBody] = React.useState('')
  //const [loading, setLoading] = React.useState(false) // 一秒钟一刷新，不能加loading

  

  const onFilterSubmit = () => queryData()

  const queryData = () => {
    let str = ''
    Object.keys(filter).map(key => { str += `${key}=${filter[key]}&` })
    str = str.slice(0, str.length - 1)
    const serverInfo = JSON.parse(localStorage.getItem('serverInfo'))
    api({ name: 'api' }).fetch(`/${serverInfo?.url3}/${podName}/logs?${str}`, {})
      .then(({ status, body }) => {
        if (status == 200) {
          setBody(body)
        }
      })
  }

  // React.useEffect(() => queryData(), [])

  return (
    <Page title={<span><ArrowLeftOutlined onClick={() => app.go('/app/model/Service/list')} /> 日志</span>}>
      <LogFilter id={id} onFilterSubmit={onFilterSubmit} filter={filter} setFilter={setFilter} />
      <LogCard body={body} />
    </Page>
  )
}

const LogFilter = ({ filter, setFilter, onFilterSubmit }) => {
  const [ifRefresh, setIfRefresh] = React.useState(true)
  const interval = React.useRef(null)
  const serverInfo = JSON.parse(localStorage.getItem('serverInfo'))

  React.useEffect(() => {
    if (ifRefresh) { // 自动刷新
      interval.current = setInterval(() => {
        onFilterSubmit()
      }, 1000)
    } else {
      clearInterval(interval.current)
    }
    return () => {
      clearInterval(interval.current)
    }
  }, [ifRefresh, filter])

  const handleChange = (values) => {
    setIfRefresh(values.refresh)
    setFilter({
      ...values,
      timestamps: values.timestamps || false,
      tail: values.tail || 100
    })
  }

  return (
    <Card title={<div style={{ display: 'flex', justifyContent: ' space-between', }}>
      <span>日志查询</span>
      <Button type="primary" onClick={onFilterSubmit}>查询</Button>
    </div>}>
      <Form
        formKey="LogFilter"
        fields={serverInfo.serverType === 'windows' ? [
          { name: 'refresh', type: 'text', label: '自动刷新', component: Switchs },
        ] : [
          { name: 'timestamps', type: 'text', label: '显示时间戳', component: Switchs },
          { name: 'since', type: 'date', label: '时间', component: DatePickerInput },
          { name: 'tail', type: 'number', label: '行数' },
          { name: 'refresh', type: 'text', label: '自动刷新', component: Switchs },
        ]}
        onChange={handleChange}
        component={FormLayout}
        onSubmit={() => { }}
        initialValues={filter}
      />
    </Card>
  )
}

const LogCard = ({ body }) => {
  return (
    <div style={{ marginTop: 10 }}>
      <Card title="日志详情">
        <div style={{ overflowX: 'auto', width: '100%', marginBottom: 10, backgroundColor: '#000' }} >
          <TextArea rows={25} value={body} style={{ backgroundColor: '#000', color: '#fff' }} />
        </div>
      </Card>
    </div>
  )
}

export default LogPage
