import React from 'react'
import { RelateAction } from 'xadmin-model/lib/relate'
import { Tag, Button, message, Tooltip, Popconfirm, Tabs, Card, Table } from 'antd'
import { ActionsBtn, ItemsBtn, DownloadBtn, UpgradeBtn } from './components/Buttons'
import { api, app, use } from 'xadmin'
import { Page, Icon, C } from 'xadmin-ui'
import { ModelBlock } from 'xadmin-model'
import { SchemaForm } from 'xadmin-form';
import moment from 'moment'
import { HasPermission } from 'xadmin-auth'
import { OnlineAddComponent, OfflineAddComponent, hignAddWinSchema } from './components/addModel'
import { CaretRightOutlined, PauseOutlined, ReloadOutlined, DeleteOutlined, PlayCircleOutlined,
  DownloadOutlined, PlusOutlined } from '@ant-design/icons';
import ReactMarkdown from 'react-markdown'

const { TabPane } = Tabs

export default {
  Service: {
    name: 'Service',
    resource: `api/win/pg/service`,
    type: 'object',
    icon: 'user', // fa-icon
    title: '服务管理',
    persistent: true,
    properties: {
      name: {
        type: 'string',
        title: '服务名称',
        column: { fixed: 'left' },
        // description: '用户的真实姓名'
        // renderTemplate: '<b style="color: <%= {1:"red", 2:"blue", 3:"green"}[item.type] || "" %>;"><%= value %></b>'
      },
      state: {
        type: 'string',
        title: '状态'
      },
      ports: {
        type: 'string',
        title: '发布端口'
      },
      version: {
        type: 'string',
        title: '当前版本'
      },
      newVersion: {
        type: 'string',
        title: '最新版本'
      }
    },
    permission: { view: true, add: true, edit: true },
    form: [],
    formEffect: form => {

      form.useEffect(({ submitSucceeded, submitErrors, submitFailed }) => {
        console.log('submitSucceeded', submitSucceeded, submitErrors, submitFailed)
      }, [ 'submitSucceeded', 'submitErrors', 'submitFailed' ])
    },
    filters: {
      // submenu: [ 'name' ],
      // sidemenu: [ 'name' ]
    },
    itemActions: [
      ({ item }) => (<ItemsBtn item={item} type="windows" />)
    ],
    dataTableProps: (columns) => ({
      scroll: { x: 1000, y: document.body.clientHeight - 210 },
      columns: columns.map(c => c.key === '__action__' ? { ...c, width: 300 } : c )
    }),
    blocks: {
      'model.list.submenu': (props) => <ActionsBtn {...props} />,
    },
    components: {
      DataForm: props => {
        // 修改页面没有名称字段
        const getEditSchema = () => {
          let result = { ...hignAddWinSchema }
          result.form = [ ...hignAddWinSchema.form.slice(1) ]
          result.required = [ ...hignAddWinSchema.required.slice(1) ]
          return result
        }
        const schema = props.id ? getEditSchema() : hignAddWinSchema

        const initialValues = props.id ? props.item : { startRetries: 3 }

        const onSubmit = value => {
          if (props.id) { // 修改
            delete value.id
            api({ name: 'api' }).fetch(`/win/pg/service/${value.name}`, {
              method: 'PUT',
              body: JSON.stringify(value)
            })
            .then(({ status }) => {
              if (status === 200) {
                message.success('修改服务成功')
                app.go('/app/model/Service/list')
              }
            }).catch(e => {
              message.error(e.message)
            })
          } else { // 新增
            api({ name: 'api' }).fetch(`/win/pg/service`, {
              method: 'POST',
              body: JSON.stringify(value)
            })
            .then(({ status }) => {
              if (status === 200) {
                message.success('新增服务成功')
                app.go('/app/model/Service/list')
              }
            }).catch(e => {
              message.error(e.message)
            })
          }
        }
        const onCloseModel = () => {}
        return props.id ? (
          <Card>
            <SchemaForm
              key="service.params.form"
              formKey="service.params"
              schema={schema}
              style={{ position: 'absolute' }}
              initialValues={initialValues}
              onSubmit={onSubmit}
              onClose={onCloseModel}
              // component={FormLayout} 
            />
          </Card>
        ) : (
          <Card>
            <Tabs defaultActiveKey="1" >
              <TabPane tab="在线添加" key="1">
                <OnlineAddComponent />
              </TabPane>
              <TabPane tab="离线添加" key="2">
                <OfflineAddComponent />
              </TabPane>
              <TabPane tab="高级添加" key="3">
                <SchemaForm
                  key="service.params.form"
                  formKey="service.params"
                  schema={schema}
                  style={{ position: 'absolute' }}
                  initialValues={initialValues}
                  onSubmit={onSubmit}
                  onClose={onCloseModel}
                  // component={FormLayout} 
                />
              </TabPane>
            </Tabs>
          </Card>
        )
        // return (<C is="Model.TabsForm" schemaKey="service.schema" forceTabs={true} baseTitle="输入并创建" {...props} />)
      },
      ListPage: props => {
        const [loading, setLoading] = React.useState(false)
        const { modelDispatch } = use('model')

        const operateAll = (name) => {
          setLoading(true)
          api({ name: 'api' }).fetch(`/win/pg/${name}`, {
            method: 'POST',
          }).then(({ status }) => {
            if (status === 200) {
              message.success('操作成功')
              modelDispatch({ type: 'GET_ITEMS' })
            } else {
              message.error('操作失败')
            }
            setLoading(false)
          }).catch(e => {
            message.error('操作失败')
            console.error(e)
            setLoading(false)
          })
        }
        return (
          <Page title="服务管理"
            subTitle={<ModelBlock name="model.list.nav" />}
            actions={<>
              <Tooltip title="启动全部服务">
                <Button loading={loading} onClick={() => operateAll('start')} >
                  <CaretRightOutlined /> 启动
                </Button>
              </Tooltip>
              <Tooltip title="重启全部服务">
                <Button loading={loading} onClick={() => operateAll('restart')} >
                  <ReloadOutlined /> 重启
                </Button>
              </Tooltip>
              <Tooltip title="停止全部服务">
                <Button loading={loading} onClick={() => operateAll('stop')} >
                  <PauseOutlined /> 停止
                </Button>
              </Tooltip>
              <Button type="primary" onClick={() => app.go('/app/model/Service/depolyment')} >
                编辑部署文件
              </Button>
              <Button type="primary" onClick={() => app.go('/app/model/Service/add')} >
                <PlusOutlined /> 添加服务
              </Button>
              <ModelBlock name="model.list.navbtn" />
            </>}
          >
            <ModelBlock name="model.list.submenu" />
            <C is="Model.DataTable" size="small" />
          </Page>
        )
      }
    },
    fieldRender: {
      state: ({ value }) => {
        let keyList = {
          'running': { title: '运行', color: 'blue' },
          'stopped': { title: '停止', color: '#a35f68' },
          'fatal': { title: '启动失败', color: 'red' },
          'retry wait': { title: '等待重试', color: 'yellow' },
          'stopping': { title: '停止中', color: 'green' },
        }
        return <Tag color={keyList[value]?.color}>{keyList[value]?.title}</Tag>
      },
      newVersion: ({ item }) => {
        let newVersion = '', description = ''
        if (app.context.newVersions) {
          app.context.newVersions.forEach(item1 => {
            if ((item1.group_name === 'server' || item1.group_name === 'driver') && item1.repo_name === item.repo) {
              newVersion = item1.releases?.[0]?.tag_name
              description = item1.releases?.[0]?.description
            }
          })
        }
        let newPackage = {}
        if (app.context.newVersions) {
          app.context.newVersions.forEach(item1 => {
            if ((item1.group_name === 'server' || item1.group_name === 'driver') && item1.repo_name === item.repo) {
              newPackage = item1.releases?.[0]?? {}
            }
          })
        }
        return (<>
          <Tooltip 
            overlayClassName="newVersion" 
            placement="left"
            color="white"
            title={() => <ReactMarkdown>{description}</ReactMarkdown>}
          ><span>{newVersion} </span></Tooltip>
          <DownloadBtn {...item} newPackage={newPackage} /> 
          <UpgradeBtn {...item} newPackage={newPackage} />
        </>)
      }
    },
    // batchActions: null,
    // editableFields: ['name', 'type', 'address.street'],
    batchChangeFields: ['website', 'brithday', 'address.street'],
    searchFields: [ 'name' ],
    required: [ 'name' ],
    readonly: [ 'id' ],
    listFields: [ 'name', 'state', 'version', 'newVersion', 'ports' ],
    defaultPageSize: 20
  }
}
