import React from 'react'
import { use, app, api } from 'xadmin'
import { message } from 'antd'
import { Model } from 'xadmin-model'
import { Page, C } from 'xadmin-ui'

const UserChangePassword = ({ context: { _t } }) => ({ 
  type: 'object',
  name: 'user_change_password',
  resource: 'user/password',
  title: _t('Change Password'),
  properties: {
    oldPassword: {
      title: _t('Old Password'),
      type: 'string',
      maxLength: 12,
      minLength: 6
    },
    newPassword: {
      title: _t('New Password'),
      type: 'string',
      maxLength: 12,
      minLength: 6
    },
    newPassword2: {
      title: _t('Repeat Password'),
      type: 'string',
      maxLength: 12,
      minLength: 6
    }
  },
  required: [ 'oldPassword', 'newPassword', 'newPassword2' ],
  form: [ 
    {
      key: 'oldPassword',
      attrs: { type: 'password' },
      validate: value => (/^(?![^a-zA-Z]+$)(?!\D+$)/ig.test(value) ? null : '密码必须为字母与数字组合')
    },
    {
      key: 'newPassword',
      attrs: { type: 'password' },
      validate: value => (/^(?![^a-zA-Z]+$)(?!\D+$)/ig.test(value) ? null : '密码必须为字母与数字组合')
    },
    {
      key: 'newPassword2',
      attrs: { type: 'password' },
      validate: (value, { newPassword }) => value == newPassword ? ((/^(?![^a-zA-Z]+$)(?!\D+$)/ig.test(value) ? null : '密码必须为字母与数字组合')) : '两次输入密码不一致'
    } 
  ]
})
const ChangePassword = props => {
  const onChange = (values) => {
    api({ name: 'api/current/password' }).fetch('', {
      method: 'PUT',
      body: JSON.stringify(values),
    }).then(({ json }) => {
      message.success('密码修改成功')
    }).catch(e => {
      message.error(e.message)
    })
  }
  return (
    <C is="Model.DataForm"
      onSubmit={onChange}
      submitText={'Change Password'}
      component={C('Auth.ChangePassword') || C('Auth.Form')}
    />
  )
}

const ChangePasswordpage = props => {
  return (
    <Page title={'修改密码'}>
      <Model schema={UserChangePassword(app)}>
        <ChangePassword {...props} />
      </Model>
    </Page>
  )
}


export default ChangePasswordpage