import { Button, Card, Form, Row, Checkbox, Input, message } from 'antd';
import _ from 'lodash';
import moment from 'moment';
import React, { useEffect } from 'react';
import { api, app, use } from 'xadmin';
import { SchemaForm } from 'xadmin-form';
import { C, Icon } from 'xadmin-ui';
import logo from './../icon/logo.svg'

//记住密码
const Remember = (props) => {
  const { input: { onChange, value } } = props
  return (
    <div className="remember-box" style={{ display: 'flex' }}>
      <Checkbox checked={value} onChange={(e) => onChange(e.target.checked)} className="remember-pwd">记住我</Checkbox>
    </div>
  )
}

//登录
const loginForm = {
  type: 'object',
  properties: {
    username: {
      type: 'string',
      title: '用户名'
    },
    password: {
      title: '密码',
      type: 'string',
      maxLength: 30,
      // minLength: 6
    },
    remenber: {
      // title: '记住我',
      // type: 'boolean'
    }
  },
  required: [ 'username', 'password' ],
  form: [ 'username', {
    key: 'password', attrs: { type: 'password' },
    // validate: value => (/^(?![^a-zA-Z]+$)(?!\D+$)/ig.test(value) ? null : '密码必须为字母与数字组合')
  },
  { key: 'remenber', component: Remember }
  ]
}

const LogoComponent = (props) => {
  return (
    <>
      <img src={logo} className="guideImg"></img>
      <h3 className="guideTitle" >AIRIOT运维管理平台</h3>
    </>
  )
}

const DefaultLayout = props => {
  const { children, handleSubmit, submitting } = props;
  const { _t } = app.context;
  return (
    <Form onSubmit={handleSubmit} layout="vertical">
      <Card style={{ marginTop: 0, borderTop: 'none', color: '#333' }}>
        {children}
        <Row>
          <Button type="primary" loading={submitting} className="loginBtn" onClick={handleSubmit} htmlType="submit">{_t('Login')}</Button>
        </Row>
      </Card>
    </Form>
  );
}

const SimpleGroup = ({ label, meta, input, field, children }) => {
  const attrs = field.attrs || {}
  const error = meta.touched && (meta.error || meta.submitError)
  const extra = field.description || field.help
  const groupProps = { extra, required: field.required }

  if (error) {
    groupProps['validateStatus'] = 'error'
    groupProps['hasFeedback'] = true
    groupProps['help'] = error
  }

  const controlComponent = children ? children : (<Input {...input} {...attrs} />)
  return (
    <Form.Item label={label} {...groupProps}>
      {controlComponent}
    </Form.Item>
  )
}

const Login = props => {
  const { dispatch } = use('redux')

  const onSubmit = (value) => {
    const data = { code: value.code, password: value.password, username: value.username }
    return new Promise((resolve, reject) => {
      api({ name: 'api/login' })
        .fetch('', {
          method: 'POST',
          body: JSON.stringify(data),
        })
        .then(({ json }) => {
          //登录成功
          let data = { ...json, username: value.username }
          if (value && value.remenber) {
            // const localValue = { ...data, time: moment().unix() }
            // localStorage.setItem('user', JSON.stringify(localValue))
            localStorage.setItem('resetCode', false)
          }
          let busLocal = {
            ...data, time: moment().unix()
          }
          localStorage.setItem('bus', JSON.stringify(busLocal))
          localStorage.setItem('user', JSON.stringify(data))

          dispatch({ type: '@@xadmin/AUTH_SIGN_IN', payload: data })
          dispatch({ type: '@@xadmin/GET_USER_INFO', payload: data, success: true })
          dispatch({ type: '@@xadmin/LOAD_VARIABLE' })

          app.go('/app/')

          resolve()
        })
        .catch((err) => {
          message.error(err.message || '登陆失败')
          localStorage.setItem('resetCode', new Date())
          if (err.status == 400 || err.status == 451) {
            reject({
              ...err.json,
              username: err.json && err.json.user,
              'FORM_ERROR': err.json && err.json.user
            })
          } else {
            err.message && Object.values(JSON.parse(err.message)).map(item=> { message.error(item) })
            reject(err)
          }
        })
    })
  }

  return (
    <div className="setGuideCard">
      <div className="guideContent" style={{ width: '648px', margin: '0 auto', borderRadius: '5px'}}>
        <div style={{ maxWidth: '688px', padding: '28px' }}>
          <div style={{ maxWidth: '648px', backgroundColor: '#fff', padding: '28px'}}>
            <div className="loginGuide" >
              <div className="loginBox">
                <LogoComponent></LogoComponent>
              </div>
              <SchemaForm
                formKey={'setLoginGuide'}
                schema={loginForm}
                component={DefaultLayout}
                group={SimpleGroup}
                onSubmit={onSubmit}
              />
            </div>
          </div>
        </div>
      </div>
    </div>
  )
}

export default Login
