import React from 'react'
import { Card, Button, Row, Form, message } from 'antd'
import { SchemaForm } from 'xadmin-form'
// import { regForm } from '../schema'
import { Icon, C } from 'xadmin-ui';
import { app, use, api } from 'xadmin'
// import LogoComponent from './LogoComponent'


const DefaultLayout = props => {
  const { children, invalid, handleSubmit, submitting } = props;
  const { _t } = app.context;
  return (
    <Form onSubmit={handleSubmit} layout="vertical">
      <Card title="注册管理员账号" style={{maxWidth: "450px",margin: "5rem auto -0.5rem"}}>
        {children}
        <Row>
          <Button type="primary" loading={submitting} className="loginBtn" onClick={handleSubmit} htmlType="submit" disabled={invalid}>{_t('Sign Up')}</Button>
        </Row>
      </Card>
    </Form>
  );
}

const schema = {
  type: 'object',
  properties: {
    password: {
      type: 'string',
      title: '密码',
      maxLength: 30,
      minLength: 6
    },
    password1: {
      type: 'string',
      title: '再次输入密码',
      maxLength: 30,
      minLength: 6
    }
  },
  form: [
    {
      key: 'password', attrs: { type: 'password' },
      validate: value => (/^(?![^a-zA-Z]+$)(?!\D+$)/ig.test(value) ? null : '密码必须为字母与数字组合')
    },
    {
      key: 'password1', attrs: { type: 'password' },
      validate: value => (/^(?![^a-zA-Z]+$)(?!\D+$)/ig.test(value) ? null : '密码必须为字母与数字组合')
    }
  ],
  required: [ 'password', 'password1' ]
}

const Reg = props => {
  // const { showCode, showExtra, onSubmit } = use('auth.reg', props)
  const onSubmit = (arg)=>{
    if (arg.password !== arg.password1) {
      message.warning('两次输入密码必须相同')
      return
    }
    return api({name: 'api'}).fetch('/user/admin',{
      method: 'POST',
      body: JSON.stringify({ password: arg.password })
    }).then(data=>{
      message.success("管理员账号注册成功")
      app.go('/login')
    },err=>{
      message.error("管理员账号已存在")
      console.error(err)
    })
  }
  return (
    <div className="setGuideCard" >
      <div className="guideContent" style={{ width: '648px', margin: '0 auto', borderRadius: '5px'}} >
        <div className="loginGuide" >
          {/* <div className="loginBox">
            <LogoComponent></LogoComponent>
            <p className="guideUse">注册管理员账号</p>
          </div> */}
          <SchemaForm
            formKey={'setRegGuide'}
            schema={schema}
            component={DefaultLayout}
            group={C('Form.SimpleGroup')}
            onSubmit={(val)=>onSubmit(val)}
          />
          <div className="loginBox">
            <p className="goReg goLogin" onClick={()=>app.go('/login')}>已有账号，去登录-> <a href="/login">立即登录</a></p>
          </div>
        </div>
      </div>
    </div>
  )
}

export default Reg
